//
// C++ Implementation: qbioritmi
//
// Description: 
//
//
// Author: Flameeyes <dgp85@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include <qpicture.h>
#include <qpainter.h>

#include <kconfig.h>
#include <kglobal.h>

#include "qbioritmi.h"
#include "bioritmo.h"

QBioritmi::QBioritmi(QWidget *parent, const char *name)
 : QLabel(parent, name)
{
	ksConfig = KGlobal::config();
}

QBioritmi::QBioritmi(KConfig *ksc, QWidget *parent, const char *name)
 : QLabel(parent, name)
{
	ksConfig = ksc;
	p.begin(&graph_p);
}

QBioritmi::~QBioritmi()
{
}

void QBioritmi::prepareGraph()
{
	H = height() -4;
	W = width();
	
	p.begin(&graph_p);
	p.setPen(QPen(QColor(0,0,0), 1));
	p.setBrush(Qt::white);
	p.drawRect(0, 2, width(), H);
	p.drawLine(0, (H/2)+2, width(), (H/2)+2);
	p.drawLine(W/2, 2, W/2, H);

	ksConfig->setGroup("colors");
		static const QColor
			scEmotional (0, 0, 127),
			scPhisical (0, 132, 0),
			scIntellectual (170, 0, 0);
			
		colorEmotional		= ksConfig->readColorEntry("emotional", &scEmotional);
		colorIntellectual	= ksConfig->readColorEntry("intellectual", &scIntellectual);
		colorPhisical		= ksConfig->readColorEntry("phisical", &scPhisical);
	
	ksConfig->setGroup("display");
		if ( ksConfig->readBoolEntry("ticks") )
		{
			p.setPen(QPen(QColor(0,0,0), 1));
			for(int i = 0; i < 30; i++)
				p.drawLine( (i*W)/30, (H/2)-1, (i*W)/30, (H/2)+5 );
		}
}

void QBioritmi::closeGraph()
{
	p.end();
	setPicture(graph_p);
}

void QBioritmi::paintBioritmi(const QDate &birdate, const QDate &centraldate)
{
	prepareGraph();
	signed char array[30][3];
	getBioArray(array, birdate, centraldate);

	ksConfig->setGroup("display");
	
		char graphs =
			(ksConfig->readBoolEntry("showEmotional", true)	  ? cyEmotional : 0x00) +
			(ksConfig->readBoolEntry("showPhisical", true)	  ? cyPhisical : 0x00) +
			(ksConfig->readBoolEntry("showIntellectual", true) ? cyIntellectual : 0x00);
	
		switch( ksConfig->readNumEntry("graphType", 0) )
		{
			case 1:
				drawLine(array, graphs);
				break;
			default:
				drawPoints(array, graphs);
				break;
		}
		
	closeGraph();
}

void QBioritmi::couple(char c, const QDate &birth1, const QDate &birth2, const QDate &centraldate)
{
	signed char array1[30][3];
	signed char array2[30][3];
	
	getBioArray(array1, birth1, centraldate);
	getBioArray(array2, birth2, centraldate);
	
	prepareGraph();
	
	static const QColor firstcolor(0,0,255), secondcolor(255,0,255);
	QColor *color;
	
	switch(c)
	{
		case cyEmotional:
			color = &colorEmotional;
			break;
		case cyPhisical:
			color = &colorPhisical;
			break;
		case cyIntellectual:
			color = &colorIntellectual;
			break;
		default:
			color = &colorMean;
	}
	
	ksConfig->setGroup("display");
		switch( ksConfig->readNumEntry("graphType", 0) )
		{
			case 1:
				KGlobal::config()->setGroup("coupler");
				*color = KGlobal::config()->readColorEntry("color1", &firstcolor);
				drawLine(array1, c);
				*color = KGlobal::config()->readColorEntry("color2", &secondcolor);
				drawLine(array2, c);
				break;
			default:
				KGlobal::config()->setGroup("coupler");
				*color = KGlobal::config()->readColorEntry("color1", &firstcolor);
				drawPoints(array1, c);
				*color = KGlobal::config()->readColorEntry("color2", &secondcolor);
				drawPoints(array2, c);
				break;
		}

	closeGraph();
}

static inline int xFromIdx(int W, int idx, int graphType)
{
	switch(graphType)
	{
		case 1:
			return (idx*W)/29;
		default:
			return (idx*W)/30;
	}
}

static inline int yFromVal(int H, signed char val)
{
	int h = H - H/20;
	return ( (h*(100+val))/200 )+4;
}

void QBioritmi::drawPoints(signed char array[30][3], char graphs)
{
	p.setPen( NoPen );
	for(int i = 0; i < 30; i++)
	{
		if ( graphs & cyEmotional ) {
			p.setBrush(colorEmotional);
			p.drawEllipse(xFromIdx(W, i, 0), yFromVal(H, array[i][0]), H/20, H/20);
		}

		if ( graphs & cyPhisical ) {
			p.setBrush(colorPhisical);
			p.drawEllipse(xFromIdx(W, i, 0), yFromVal(H, array[i][1]), H/20, H/20);
		}

		if ( graphs & cyIntellectual ) {
			p.setBrush(colorIntellectual);
			p.drawEllipse(xFromIdx(W, i, 0), yFromVal(H, array[i][2]), H/20, H/20);
		}

		if ( graphs & cyMean ) {
			signed char mean = (array[i][0] + array[i][1] + array[i][3]) /3;
			p.setBrush(colorMean);
			p.drawEllipse(xFromIdx(W, i, 0), yFromVal(H, mean), H/20, H/20);
		}
	}
}

void QBioritmi::drawLine(signed char array[30][3], char graphs)
{
	for(int i = 0; i < 29; i++)
	{
		if ( graphs & cyEmotional ) {
			p.setPen(QPen(colorEmotional, 2));
			p.drawLine(xFromIdx(W, i, 1), yFromVal(H, array[i][0]), xFromIdx(W, i+1, 1), yFromVal(H, array[i+1][0]));
		}

		if ( graphs & cyPhisical ) {
			p.setPen(QPen(colorPhisical, 2));
			p.drawLine(xFromIdx(W, i, 1), yFromVal(H, array[i][1]), xFromIdx(W, i+1, 1), yFromVal(H, array[i+1][1]));
		}

		if ( graphs & cyIntellectual ) {
			p.setPen(QPen(colorIntellectual, 2));
			p.drawLine(xFromIdx(W, i, 1), yFromVal(H, array[i][2]), xFromIdx(W, i+1, 1), yFromVal(H, array[i+1][2]));
		}
		
		if ( graphs & cyMean ) {
			signed char mean1 = (array[i][0] + array[i][1] + array[i][2]) /3,
				    mean2 = (array[i+1][0] + array[i+1][1] + array[i+1][2]) /3;
			p.setPen(QPen(colorMean, 2));
			p.drawLine(xFromIdx(W, i, 1), yFromVal(H, mean1), xFromIdx(W, i+1, 1), yFromVal(H, mean2));
		}
	}
}

void QBioritmi::getBioArray(signed char array[30][3], const QDate &birthdate, const QDate &origin)
{
	bioritmo bio;
	bio.set_bir_date(birthdate);

	QDate d = origin.addDays(-14);
	for(int i = 0; i < 30; i++)
	{
		bio.set_bio_date(d);
		
		array[i][0] = bio.cEmotivo();
		array[i][1] = bio.cFisico();
		array[i][2] = bio.cIntellettuale();
		
		d = d.addDays(1);
	}
}

#include "qbioritmi.moc"
